proc PDFileMenus { fileMenu fileVar args }  {

    set emptyValue "**NO FILES**"
    set filter "*.*"
    set pathVar ""
    set pathMenu ""
    set path [pwd]
    set initialPath [pwd]
    set filenameDisplay standard
    set pathnameDisplay relative
    set maxDepth -1

    foreach {key value} $args {
	switch -exact -- $key {

	    -pathMenu		{ set pathMenu $value }
	    -pathVar		{ set pathVar $value }
	    -filter		{ set filter  $value }
	    -emptyValue		{ set emptyValue $value }
	    -path		{ set path $value }
	    -maxDepth		{ set maxDepth $value }
	    -filenameDisplay	{ set filenameDisplay $value }
	    -pathnameDisplay	{ set pathnameDisplay $value }
	}
    }
    
    
    if [catch {cd "$path"} err] {
	error "Could not find the directory $path: $err"
	return
    }

    puts "In $path....getting Files"

    set filesAndDirs  [__PDFMGetFilesAndDirs [pwd] $filenameDisplay \
			   $pathnameDisplay $filter $maxDepth]

    set files [lindex $filesAndDirs 0]
    set dirs [lindex $filesAndDirs 1]
    set fulldirs [lindex $filesAndDirs 2]

    if { ![winfo exists $fileMenu] } {	

	if { $files == {} } {
	    tk_optionMenu $fileMenu $fileVar $emptyValue
	    $fileMenu configure -state disabled
	} else {
	    eval tk_optionMenu $fileMenu $fileVar $files
	}
    
    } else {
	error "$fileMenu already exists."
	if [catch {cd "$initialPath"} msg] {
	    error "Could not return to the original path '$initialPath': $msg"
	}
	return
    }
    
    if { [string compare $pathMenu ""] != 0 && \
	     [string compare $pathVar ""] != 0 } {
	# create the path menu as well...

	if { ![winfo exists $pathMenu] } {
	    eval tk_optionMenu $pathMenu __PDFMInternal$pathVar $dirs
	    uplevel #0 set $pathVar [lindex $fulldirs 0]
	    set dirLen [llength $dirs]
	    for {set i 0} {$i < $dirLen } {incr i} {
		[$pathMenu cget -menu] entryconfigure $i -command \
		    [list __PDFMLoadFiles [pwd] [lindex $fulldirs $i] \
			 $fileMenu $fileVar $filenameDisplay $pathMenu \
			 $pathVar $pathnameDisplay $filter $maxDepth \
			 $emptyValue]
	   }
 
	} else {
	    error "$pathMenu already exists."
	}
    } else {
	# We want to keep track of the path anyway
	uplevel #0 set __PDFMInternal$pathVar "$path"
    }
    if [catch {cd "$initialPath"} msg] {
	error "Could not return to the original path '$initialPath': $msg"
    }
    
    return
}


proc __PDFMLoadFiles {topDir workingDir fileMenu fileVar filenameDisplay \
			  pathMenu pathVar pathnameDisplay filter \
			  maxDepth emptyValue } {

    set cPath [pwd]
    cd "$workingDir"
    

    set filesAndDirs [__PDFMGetFilesAndDirs $topDir $filenameDisplay \
			  $pathnameDisplay $filter $maxDepth]

    [$pathMenu cget -menu] delete 0 end
    [$fileMenu cget -menu] delete 0 end
    
    set files [lindex $filesAndDirs 0]
    set dirs [lindex $filesAndDirs 1]
    set fulldirs [lindex $filesAndDirs 2]

    set nDirs [llength $dirs]
    
    if { $files == {} } {
	uplevel #0 [list set $fileVar "$emptyValue"]
	$fileMenu configure -state disabled
    } else {
	foreach file $files {
	    [$fileMenu cget -menu] add command -label $file -command \
		[list set $fileVar $file]
	}
	uplevel #0 set $fileVar [lindex $files 0]
	$fileMenu configure -state normal
    }

    
    for {set i 0} {$i < $nDirs} {incr i} {
	[$pathMenu cget -menu] add command -label [lindex $dirs $i] \
	    -command [list __PDFMLoadFiles $topDir [lindex $fulldirs $i] \
			 $fileMenu $fileVar $filenameDisplay $pathMenu \
			 $pathVar $pathnameDisplay $filter $maxDepth \
			 $emptyValue ]
    }
    uplevel #0 set __PDFMInternal$pathVar \"[lindex $dirs 0]\"
    uplevel #0 set $pathVar \"[lindex $fulldirs 0]\"

    cd "$cPath"
    
}
    
proc __PDFMGetFilesAndDirs { topDir filenameDisplay \
			     pathnameDisplay filter maxDepth} {



    set currDir [pwd]
    if { $maxDepth < 0 } {
	set cDepth 1
    } else {
	set cDepth [expr $maxDepth - [llength [file split $currDir]] \
			      + [llength [file split $topDir]]]
    }
   

    switch -exact $filenameDisplay {
	
	standard	{ 
	    foreach i [glob -nocomplain $filter] { 
		lappend files [file tail $i] 
	    }
	}
	root		{ 
	    foreach i [glob -nocomplain $filter] { 
		lappend files [file rootname [file tail $i]] 
	    }
	}
	default		{
	    set files [glob -nocomplain $filter]
        }
    }
    
    switch -exact $pathnameDisplay {
	base		{
	    if { $topDir == $currDir } {
		set dirs .
		set fulldirs $topDir
	    } else {
		set dirs [list . .. ]
		set fulldirs [list $currDir [file join $currDir ..]]
	    }
	    if { $cDepth > 0 } {
		foreach file [glob -nocomplain *] {
		    if [file isdirectory $file] {
			lappend dirs [file join . [file tail $file]]
			lappend fulldirs [file join $currDir $file]
		    }
		}
	    }
	}
	relative        {
	    if { $topDir == $currDir } {
		set thisPath .
		set dirs .
		set fulldirs $topDir
	    } else {
		set thisPath [eval file join . [lrange [file split $currDir] \
					       [llength [file split $topDir]] \
						     end]]
		set dirs [list $thisPath [file join $thisPath ..]]
		set fulldirs [list $currDir [file join $currDir ..]]
	    } 
	    if { $cDepth > 0 } {
		foreach file [glob -nocomplain *] {
		    if [file isdirectory $file] {
			lappend dirs [file join $thisPath $file]
			lappend fulldirs [file join $currDir $file]
		    }
		}
	    }
		
	}
	default		{
	    if { $topDir == $currDir } {
		set dirs $currDir
		set fulldirs $currDir
	    } else {
		set dirs [list $currDir [file join $currDir ..]]
		set fulldirs [list $currDir [file join $currDir ..]]
	    }
	    if { $cDepth > 0 } {
		foreach file [glob -nocomplain *] {
		    if [file isdirectory $file] {
			lappend dirs $file
			lappend fulldirs [file join $currDir $file]
		    }
		}
	    }
	}
    }

    set topDir [pwd]
    

    
    if [info exists files] { 
	return [list [lsort -ascii $files] $dirs $fulldirs]
    } else {
	return [list {} $dirs $fulldirs]
    }
    
}


