#### -*- Mode: Emacs-Lisp -*- 
###############################################################################
####              
#### File            : tsiAgentWindow.tcl
#### Authors         : Randy Jones, etc.
#### Created On      : not known
#### Last Modified By: Frank Ritter
#### Last Modified On: 16-Feb-97
#### Update Count    : 
#### 
#### PURPOSE
#### This script implements a GUI for a Soar application.
####
#### TABLE OF CONTENTS
####
####	i.	Source File INCLUDEs and globals
####	I.	Interaction window Definition
#### 	II.	Support PROCEDURES
####
###############################################################################
global tsiConfig

###
###	I.	Interaction window Definition
###

proc makeSoarInteractionWindow {{x 0} {y 30} } {
  global tsiConfig auto_path tsi_library
  global wwwViewer soar_library interp_name

  if $tsiConfig(debug) {puts "\n>>> starting makeSoar interaction window1"}

  if $tsiConfig(debug) {
    puts "\n>>> starting makeSoar interaction window2 $interp_name"
  }

  if [catch registerWithController] {
      error "Failed to register new agent with controller"
  }

  if $tsiConfig(debug) {puts "\n>>> starting makeSoar interaction window3"}
  # if the window already exists, just quit

  if {[winfo exists .tsw] != 0} { return }

  if $tsiConfig(debug) {puts "\n>>> starting makeSoar interaction window2"}
  # Otherwise build a new top-level window

  toplevel .tsw
  wm title .tsw "Interaction Window: Agent $interp_name"
  wm iconname .tsw $interp_name
 
  if {$x >= 0} { set x +$x }
  if {$y >= 0} { set y +$y }
  wm geometry .tsw $x$y

  # Create menus that will be attached to menu bar.  We start by defining
  # a frame to contain all the menu buttons.

  frame .tsw.menubar -relief raised -bd 2
  pack .tsw.menubar -side top -fill x

  #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  # The following menus are listed in the order they appear in the menu
  # bar (except for the "Demos" menu since it is packed to the right
  # instead of the left).  Within menu sections, commands are listed
  # in the order of the menu entries.  Cascades and sub-cascades are
  # listed separately.

  addTSIFileMenu .tsw

  addTSIShowMenu .tsw

  addTSIMemoryMenu .tsw

  addTSIProductionMenu .tsw

  if $tsiConfig(debug) {puts "\n>>> starting makeSoar interaction window-watch"}
  addTSIWatchMenu .tsw

  if $tsiConfig(debug) {puts "\n>>> starting makeSoar interaction window-monitor"}
  addTSIMonitorMenu .tsw

  if $tsiConfig(debug) {puts "\n>>> starting makeSoar interaction window-commands"}
  addTSICommandsMenu .tsw

  if $tsiConfig(debug) {puts "\n>>> starting makeSoar interaction window-help"}
  addTSIHelpMenu .tsw

#  if $tsiConfig(debug) {puts "\n>>> starting makeSoar interaction window-demo"}
#  addTSIDemoMenu .tsw

  #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  # Create TEXT WINDOW and scrollbar

  if $tsiConfig(debug) {puts "\n>>> starting makeSoar interaction window-term"}

  setUpSoarTerm .tsw.t
  .tsw.t configure -font $tsiConfig(normalFont)
  .tsw.t configure -yscrollcommand {.tsw.s set}
  scrollbar .tsw.s -relief flat -command {.tsw.t yview}

  # QUICK ACTION BUTTONS

  frame .tsw.frame -borderwidth 0
  button .tsw.frame.run  -text {Run} -command {tsiDisplayAndSendCommand run}
  button .tsw.frame.step -text {Step} -command {tsiDisplayAndSendCommand {run 1}} 
  button .tsw.frame.stop -text {Stop} -command {tsiDisplayAndSendCommand {stop}}
  button .tsw.frame.search -text {Search} -command {searchWindow .tsw}

  # Pack up the GUI

  pack .tsw.s -side right -fill both
  pack .tsw.t -side top -expand 1 -fill both

  pack .tsw.frame.step .tsw.frame.run .tsw.frame.stop -side left
  pack .tsw.frame.search -side right
  pack .tsw.frame -side bottom -fill x

  #--------------------------------------------------------------------------

  # BINDINGS

  # Don't like "click to focus"
  bind .tsw <Enter> { focus .tsw.t }

  #--------------------------------------------------------------------------
  # Startup INITIALIZATIONS:

  # Redirect all printed output (from stdout, usually) to the text widget

  ## currently, 1-Feb-97, the mac can't handle the extra widget, so keep 
  ## output at default location
  ##global tcl_platform
  ##if {$tcl_platform(platform) != {mac}} {
  ##}
  # This better work in Soar 7.1 or we are in trouble RMJ
  #output-strings-destination -push -text-widget .tsw.t
  output-strings-destination -push -procedure tsiOutputToWidget

  # We add a monitor to make sure that long running commands like
  # "go" do not stop X events from being processed.

  monitor -add after-decision-cycle update

  eval .tsw.t tag configure prompt $tsiConfig(promptConfigure)
  eval .tsw.t tag configure oldUserText $tsiConfig(userTextConfigure)
  eval .tsw.t tag configure searchFind $tsiConfig(searchTextConfigure)

  # Initialize focus of keyboard events
  focus .tsw.t
}

if $tsiConfig(debug) {puts "\n>>> tsiAgentWindow1"}

#--------------------------------------------------------------------------



###
### 	II.	Support PROCEDURES
### 
proc tsiOutputToWidget {x} {
   .tsw.t insert end $x
   .tsw.t see end
}

######## Set the watch options from a menu
proc resetWatch {attribute {on -on} {off -off}} {
   global watchValue
   if $watchValue($attribute) {
      tsiDisplayAndSendCommand [concat watch $attribute $on]
   } else {
      tsiDisplayAndSendCommand [concat watch $attribute $off]
   }
}

proc tsiSetWMELevel {} {
   global watchValue
   switch $watchValue(WMElevel) {
      1 {tsiDisplayAndSendCommand {watch -nowmes}}
      2 {tsiDisplayAndSendCommand {watch -timetags}}
      3 {tsiDisplayAndSendCommand {watch -fullwmes}}
   }
}

## Set up the directories for later use in soar.soar
## Notes: different paths, can't use ::: in the midst of a cd, just ::
## different types of files as well, and can't spawn html viewer as easily
## get an error:>> invalid command name ".showfile1.text"<<

######## New definition of help

### Currently, the "help" command in Soar sends its output directly to
### the "console" (for lack of a better term, that is wherever Soar was
### run from), with no way to redirect it (to a text widget, for example).
### So for the graphical interface, if someone types "help" we need to
### trap the command and do our own version where we load the help file
### into a text widget ourselves.

### The regular soar "help" command is renamed in tsiInitAgent
proc help args {
    global soar_library soar_help tcl_version soar_doc_dir
    ### If the user types "help" with no arguments, or with the "-usage"
    ### argument, we can simply pass the command on to soar (using
    ### tsiInternalHelp), and the output will be directed to the text widget.
    ### The "help -usage" output has formatting commands in it, so we will
    ### pass that through the ScanTextAndHighlight procedure.
    if {($args == {}) || ([string index [lindex $args 0] 0] == {-})} {
        if {[lindex $args 0] == {-usage}} {
            scan [.tsw.t index insert] %d start}
        termTextProcessCommand .tsw.t [concat tsiInternalHelp $args]
        if {[lindex $args 0] == {-usage}} {
            scan [.tsw.t index end] %d finish
            ScanTextAndHighlight .tsw.t $start $finish }
        return
    }
    ### If we get this far, the user has asked for help on a specific topic,
    ### so we handle that case ourselves by displaying the topic file in a
    ### separate window.
    set topic [lindex $args 0]
    ## Soar-Bugs 127 - Test for unambiguous partial command
    if {([llength [info commands ${topic} ]] != 1) &&
         ([llength [info commands ${topic}*]] == 1)   } {
         set topic [info commands ${topic}*]
    }
    if [info exists soar_doc_dir] {
       set fullTopic [file join $soar_doc_dir cat $topic.n]
    } else {
       set soar_dir [file dirname $soar_library]
       set fullTopic [file join $soar_dir ../doc/cat $topic.n]
    }
    if {[file isfile $fullTopic] == 1} {
        ShowFile "$fullTopic" 0
    } else {
        .tsw.t insert insert "Sorry, there is no help on \"$topic\".\n"
    }
}

# These procedures are used to run the demos which do not have a
# GUI.

if $tsiConfig(debug) {puts "\n>>> tsiAgentWindow2"}

# Can be changed to work with mac and unix versions.
# 26-Mar-97 -FER
proc loadTSIFile {demo} {
    global tsi_library
    if [catch "pushd [file join $tsi_library demos]"] {
       set old_dir [pwd]
       cd [file join $tsi_library demos]
    }
    tsiDisplayAndSendCommand {excise -all}
    tsiDisplayAndSendCommand "source $demo"
    if [info exists old_dir] {
       if [catch "cd $old_dir"] {
          puts "Warning: Couldn't return to directory $old_dir"
       }
    } elseif [catch popd] {
       puts {Warning: Couldn't return to old directory}
    }
}

proc RunGUIDemo {demo} {
    global default soar_library

    tsiDisplayAndSendCommand {excise -all}
    tsiDisplayAndSendCommand "source \"[file join $soar_library default.soar]\" "
    tsiDisplayAndSendCommand "uplevel #0 [list pushd \
        \"[file join [file dirname $soar_library] demos gui \
                                  $demo]\"]"
    tsiDisplayAndSendCommand "uplevel #0 [list source $demo.tcl]"
    tsiDisplayAndSendCommand "uplevel #0 popd"
}

proc RunNonGUIDemo {demo} {
    global default soar_library

    tsiDisplayAndSendCommand {excise -all}
    tsiDisplayAndSendCommand "source $default"
    tsiDisplayAndSendCommand "source \
        [file join [file dirname $soar_library] demos no-gui \
                                  $demo $demo.soar]"
}

proc RunNonGUIProbDemo {demo prob} {
    global default soar_library

    tsiDisplayAndSendCommand {excise -all}
    tsiDisplayAndSendCommand "source $default"
    tsiDisplayAndSendCommand "source \
        [file join [file dirname $soar_library] demos no-gui \
                                  $demo $demo.soar]"
    tsiDisplayAndSendCommand "source \
        [file join [file dirname $soar_library] demos no-gui \
                                  $demo $prob]"
}

### Use the information we know about mosaic and netscape WWW viewers
### to view web pages in a friendly way (i.e., use an existing viewer
### process if one is available, otherwise start the viewer up).
proc viewWWW {url} {
   global wwwViewer env

   if ![info exists wwwViewer] {
      tk_dialog .error {Error} "WWW viewer not set.\nSet wwwViewer to mosaic, \
xmosaic, or netscape\n(other viewers can be tried but may not work)." error 0 Dismiss
      return
   }
   # mosaic/xmosaic
   if {([string first mosaic $wwwViewer] == 0) || \
       ([string first xmosaic $wwwViewer] == 0)} {
      if {![catch "set mp [open [file join $env(HOME) .mosaicpid] r]"] && \
          ![catch "gets $mp pid"]} {
         close $mp
         if ![catch "set mp [open /tmp/Mosaic.$pid w]"] {
            puts $mp newwindow
            puts $mp $url
            close $mp
            if ![catch "exec kill -USR1 $pid"] {
               return
            }
         }
      }
      if [catch "exec $wwwViewer $url &"] {
         tk_dialog .error {Error} "Cannot launch $wwwViewer WWW viewer" \
                   error 0 Dismiss
      }
      return
   }
   # netscape
   if {[string first netscape $wwwViewer] == 0} {
      if [catch "exec $wwwViewer -remote openURL($url)"] {
         if [catch "exec $wwwViewer $url"] {
         tk_dialog .error {Error} "Cannot launch $wwwViewer WWW viewer" \
                   error 0 Dismiss
         }
      }
      return
   }
   # Tcl/Tk-based viewers?
   #
   # all others
   if [catch "exec $wwwViewer $url"] {
      tk_dialog .error {Error} "Cannot launch $wwwViewer WWW viewer" \
                error 0 Dismiss
   }
}

proc addTSIProductionMenu {w} {
  
  ### Initialize variable settings for menus
  if [winfo exists $w.menubar.prod.m] {
    destroy $w.menubar.prod.m
  } else {
    menubutton $w.menubar.prod -text {Productions} -menu $w.menubar.prod.m
  }

  menu $w.menubar.prod.m -tearoff 0
  $w.menubar.prod.m add command -label {Reload User Productions} -command {addTSIProductionMenu .tsw}

  output-strings-destination -push -append-to-result
  set userProds [print -user]
  output-strings-destination -pop
 
  # To avoid memory problems, don't do over 200 productions
  set userProds [lrange $userProds 0 200]
  
  foreach v $userProds {
	set commandstr1 "tsiDisplayAndSendCommand {matches $v}"
	set commandstr2 "tsiDisplayAndSendCommand {matches $v -wmes}"
	set commandstr3 "termTextInsert $w.t $v"
	set commandstr4 "tsiDisplayAndSendCommand {print $v}"
	set firstAsterisk [string first "*" $v]
	set firstLetter [string range $v 0 [expr $firstAsterisk - 1]]
	if {![info exists alphList($firstLetter)]} {
	  $w.menubar.prod.m add cascade -label "[string toupper $firstLetter]'s" \
             -menu $w.menubar.prod.m.$firstLetter
	  menu $w.menubar.prod.m.$firstLetter -tearoff 0
	  set alphList($firstLetter) 1
	}
	  $w.menubar.prod.m.$firstLetter add cascade -label $v \
             -menu $w.menubar.prod.m.$firstLetter.$v
	  menu $w.menubar.prod.m.$firstLetter.$v -tearoff 0
	  $w.menubar.prod.m.$firstLetter.$v add command -label {matches} \
           -command $commandstr1
	  $w.menubar.prod.m.$firstLetter.$v add command -label {matches -wmes} \
           -command $commandstr2
	  $w.menubar.prod.m.$firstLetter.$v add command -label {print production name} \
           -command $commandstr3
	  $w.menubar.prod.m.$firstLetter.$v add command -label {print production} \
           -command $commandstr4
  }

  pack $w.menubar.prod -side left -padx 5

}
