###
###
### tsiControlPanelCommon.tcl
###
###
### This file contains Control Panel helper functions which do not 
### refer to any control panel specific properties, and which are 
### thought to be useful for all (or many) controllers. 
###
### The idea here is that the control panel is segmented into 2 files,
### the remaining file defines the control panel itself as well as the
### control panel specific procedures.  NOTE: these control panel specific 
### procedures MUST include:
###		registerAgent {name}
###		destroyAgent {name}



proc tsiSetControlPanelSIOMode { topFrame } {
    global tsiConfig tsi_library


  # Now we need to do the specialized setup for each particular mode

    if { ![string compare $tsiConfig(mode) client] } {
      # Client mode
      
      uplevel #0 [list source [file join $tsi_library tsiClient.tcl]]
      tsiInitClientControlPanel $tsiConfig(socketioHost) \
	  $tsiConfig(socketioPort) $topFrame

  } elseif { ![string compare $tsiConfig(mode) server] } {
      # Server mode
      
      
      uplevel #0 [list source [file join $tsi_library tsiServer.tcl]]
      tsiInitServerControlPanel $tsiConfig(socketioPort) $topFrame

  } else {
      # Socketio Off

      uplevel #0 [list source [file join $tsi_library tsiLocal.tcl]]
           
      
  }

}
proc tsiListAgents {} {
   if [catch {set x [interp slaves]} m] {
      error "Cannot compute list of agents: $m"
   }
   return $x
}

### need to worry about having whitespace in the agent name...that
### can break things RMJ
proc createNewAgent {name {filepath ""} {filename ""}} {
   global auto_path tsi_library soar_library soar_doc_dir tsiConfig \
       localAgents tsiAgentInfo

   if [catch "interp create $name"] {
      error "Cannot create agent named '$name'"
   }

    if [info exists localAgents] { 
	lappend localAgents $name 
    } else {
	set localAgents $name
    }


   load {} Tk $name
   $name eval [list set auto_path $auto_path]
   $name eval {package require Soar}
   $name eval [list set soar_library $soar_library]
   $name eval [list set tsi_library $tsi_library]
   if [info exists soar_doc_dir] {
      $name eval [list set soar_doc_dir $soar_doc_dir]
   }
   ### Configure the agent for the TSI
   set id [array startsearch tsiConfig]
   while {[array anymore tsiConfig $id]} {
      set elm [array nextelement tsiConfig $id]
      $name eval [list set tsiConfig($elm) $tsiConfig($elm)]
   }
   array donesearch tsiConfig $id
   $name eval [list set interp_name $name]
   $name eval [list set source_path $filepath]
   $name eval [list set source_file $filename]
   $name alias registerWithController registerAgent $name
   $name alias tsiListAgents tsiListAgents
   $name eval tsiInitAgent
   

   tsiSetupAgentVars $name

    set tsiAgentInfo($name,sourceDir) "$filepath"
    set tsiAgentInfo($name,sourceFile) "$filename"

    tsiLoadAgentSource $name
	
	
}


proc tsiLoadAgentSource {name} {
    global tsiAgentInfo
    
    if { [string compare $tsiAgentInfo($name,sourceDir) ""] != 0 } {
	set initialPath [pwd]
	
	if [catch {cd "$tsiAgentInfo($name,sourceDir)"} msg] {
	    error "Could not change to directory '$tsiAgentInfo($name,sourceDir)'"
	    return
	}
	if [catch "$name eval [list uplevel #0 source $tsiAgentInfo($name,sourceFile)]" msg] {
	    
    	    tk_dialog .error Warning \
                "Couldn't load agent code for $name from file '$tsiAgentInfo($name,sourceFile): $msg" warning 0 Ok
	}
	
	if [catch "cd $initialPath" msg] {
	    error "Could not change to directory '$initialPath'"
	    return
	}
	
	
	
    }

    $name eval addTSIProductionMenu .tsw
}
    

proc tsiSendAgent {agent command} {
   ### Need to use {} for catch, or $command gets concat'ed into a list
   if [catch {$agent eval $command} msg] {
      error "Error sending command '$command' to agent $agent!"
   }
}

### New version using slave interpreters in Soar 7.1
proc sendAllAgents {msg} {
   foreach i [tsiListAgents] {
      if [catch {$i eval $msg} m] {
         error "Error sending message $msg to agent $i: $m"
      }
   }
}


proc agentToggleCommand {name t on off} {
   if $t {
      tsiSendAgent $name $on
   } else {
      tsiSendAgent $name $off
   }
}


### Need a fake-stop-soar, because stop-soar is not defined in wish
### interpreters
proc fake-stop-soar {} {
   set a [tsiListAgents]
   if {$a == {}} {
      tk_dialog .error {Error} \
                       {Can't stop!  No Soar interpreters are running!}\
                error 0 Dismiss
      return
   }
   tsiSendAgent [lindex $a 0] stop-soar
}




proc toggleWindow {win} {
   
   if {[string compare [wm state $win] normal] == 0 } {
      wm withdraw $win
   } else {
      wm deiconify $win
   }
}

